@@ -1,15 +1,18 @@
-import os
 from importlib.util import find_spec
+from pathlib import Path
 from typing import Tuple
 
 import torch
+from torch import Tensor
 
 from pycarus.geometry.pcd import batchify, unbatchify
 
 
 def chamfer(
-    prediction: torch.Tensor, groundtruth: torch.Tensor, squared: bool = True
-) -> Tuple[torch.Tensor, torch.Tensor]:
+    prediction: Tensor,
+    groundtruth: Tensor,
+    squared: bool = True,
+) -> Tuple[Tensor, Tensor]:
     """Compute the Chamfer Distance defined as in:
 
     Groueix, T., Fisher, M., Kim, V. G., Russell, B. C., & Aubry, M. (2018).
@@ -37,16 +40,17 @@ def chamfer(
     batched, [pcd_src, pcd_trg] = batchify([pcd_src, pcd_trg], 3)
 
     if use_cuda:
-        not_chamfer_cuda = find_spec("chamfer_cuda") is None
+        not_chamfer_cuda = find_spec("chamfercuda") is None
 
         if not_chamfer_cuda:
-            folder = os.path.dirname(os.path.realpath(__file__))
-            str_exp = f"Chamfer Cuda module not found. Install it running {folder}/external/setup_chamfer_gpu.py"
+            pycarus_root = Path(__file__).parent.parent.absolute()
+            str_exp = "Chamfer CUDA module not found. Install it running: "
+            str_exp += f"source {pycarus_root}/install_cuda_ext.sh {pycarus_root}"
             raise ModuleNotFoundError(str_exp)
         else:
-            from pycarus.metrics.external.chamfer_gpu import Chamfer_GPU  # type: ignore
+            from pycarus.metrics.external.chamfer_cuda import Chamfer_CUDA  # type: ignore
 
-            chamfer = Chamfer_GPU()
+            chamfer = Chamfer_CUDA()
             dist_x_y, dist_y_x, _, _ = chamfer(pcd_src, pcd_trg)
     else:
         x, y = pcd_src.double(), pcd_trg.double()
@@ -71,3 +75,39 @@ def chamfer(
         mean_dist_x_y, mean_dist_y_x = unbatchify([mean_dist_x_y, mean_dist_y_x])
 
     return mean_dist_x_y, mean_dist_y_x
+
+
+def chamfer_p(prediction: Tensor, groundtruth: Tensor) -> Tensor:
+    """Compute the chamfer distance "P".
+
+    The result is computed as:
+        - cd_pred2gt, cd_gt2pred = chamfer(prediction, groundtruth, squared=False)
+        - result = (cd_pred2gt + cd_gt2pred) / 2.
+
+    Args:
+        prediction: The source point cloud(s) with shape ([B,] NUM_POINTS, 3).
+        groundtruth: The target point cloud(s) with shape ([B,] NUM_POINTS, 3).
+
+    Returns:
+        The computed chamfer distance "P".
+    """
+    cds_pred_gt, cds_gt_pred = chamfer(prediction, groundtruth, squared=False)
+    return (cds_gt_pred + cds_pred_gt) / 2
+
+
+def chamfer_t(prediction: Tensor, groundtruth: Tensor) -> Tensor:
+    """Compute the chamfer distance "T".
+
+    The result is computed as:
+        - cd_pred2gt, cd_gt2pred = chamfer(prediction, groundtruth, squared=True)
+        - result = cd_pred2gt + cd_gt2pred.
+
+    Args:
+        prediction: The source point cloud(s) with shape ([B,] NUM_POINTS, 3).
+        groundtruth: The target point cloud(s) with shape ([B,] NUM_POINTS, 3).
+
+    Returns:
+        The computed chamfer distance "T".
+    """
+    cds_pred_gt_sq, cds_gt_pred_sq = chamfer(prediction, groundtruth, squared=True)
+    return cds_pred_gt_sq + cds_gt_pred_sq
